@@ -224,6 +224,12 @@ first <- function(vec) { vec[1] }
 #   return(vec)
 # }
 
+
+# prepend to a vector
+prepend <- function(x,values,before=1)
+{ append(x,values,after=before-1) }
+
+
 # CLAMP A NUMBER
 clamp <- Vectorize(function(num,min=0,max=1) { if(num<min) {min} else if(num<max) {num} else {max} })
 
@@ -246,6 +252,7 @@ pad <- function(vec,size,padding=0,side="right")
 # row pad for data frames / matrices
 rpad <- function(mat,size,padding=0,side="right")
 {
+  mat <- cbind(mat)
   size <- size - nrow(mat)
   COL <- ncol(mat)
   padding <- array(padding,c(size,COL))
