@@ -31,9 +31,9 @@ clean.parameters <- function(par)
 
 
 # identify autocovariance parameters in ctmm object
-# if profile=TRUE, some parameters can be solved exactly and so aren't identified
-# if linear=TRUE, only return linear non-problematic parameters
-id.parameters <- function(CTMM,profile=TRUE,linear=FALSE,UERE=FALSE,dt=0,df=0)
+# if profile==TRUE, some parameters can be solved exactly and so aren't identified
+# if linear==TRUE, only return linear non-problematic parameters
+id.parameters <- function(CTMM,profile=TRUE,linear=FALSE,UERE=FALSE,dt=0,df=0,dz=10)
 {
   # identify and name autocovariance parameters
   NAMES <- NULL
@@ -99,7 +99,7 @@ id.parameters <- function(CTMM,profile=TRUE,linear=FALSE,UERE=FALSE,dt=0,df=0)
 
   if(CTMM$error && UERE<3)
   {
-    parscale <- c(parscale,max(10,CTMM$error)) # minimum of 10-meter error parscale
+    parscale <- c(parscale,max(dz,CTMM$error)) # minimum of dz error parscale
     lower <- c(lower,0)
     upper <- c(upper,Inf)
     period <- c(period,FALSE)
